/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/*!
@file main.cpp
@ingroup ExamplePrograms
@brief Obtains <a href="http://en.wikipedia.org/wiki/Scale-invariant_feature_transform">SIFT descriptors</a> over the frames from an input video source.

@todo Detailed description should follow.

@section Usage
Once compiled, the program should be used with this command line:
\code
	./siftextractor --URL=<path or url to video>
\endcode

@param URL Video source identificator. See dynamic property URL for @ref QVMPlayerCamera.
@author PARP Research Group. University of Murcia, Spain.
*/

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <QDebug>

#include <qvgui/qvgui.h>
#include <qvcameras/qvmplayercamera.h>
#include <qvcore/qvapplication.h>
#include <qvipp/qvipp.h>

#include "siftextractor.h"

////////////////////////
// Class MyWorker
class MyWorker: public QVWorker
	{
	public:
		MyWorker(QString name): QVWorker(name)
			{
			addProperty< QVImage<uChar,3> >("Input image", inputFlag|outputFlag);
			addProperty< QVImage<uChar,3> >("Sift image", outputFlag);
			}

		void iterate()
			{
			QVImage<uChar,3> image = getPropertyValue< QVImage<uChar,3> >("Input image");

			timeFlag("Image grab");

			/////////////////
			// SIFT
			QList<QVSIFTDescriptor>	descriptors = getSIFTDescriptors(image);
			timeFlag("Get SIFT descriptors");

			QVImage<uChar,3> descriptorsImg = image;
			drawSIFTDescriptors(descriptors, descriptorsImg);
			timeFlag("Draw SIFT descriptors");

			setPropertyValue< QVImage<uChar,3> >("Sift image",descriptorsImg);
			}
	};

int main(int argc, char *argv[])
	{
	QVApplication app(argc, argv,
		"Example program for QVision library. Gets SIFT descriptors from image."
		);
	
	QVMPlayerCamera camera("Video");
	MyWorker worker("SIFT Worker");
	camera.link(&worker,"Input image");

	QVGUI interface;

	QVImageCanvas imageCanvas("Sift image");
	imageCanvas.linkProperty(worker, "Sift image");

	return app.exec();
	}
