/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/*!
@file toon-example.cpp
@ingroup ExamplePrograms
@brief Example program for using TooN library in a QVision program.

@todo Detailed description should follow.
@section AboutTooN About TooN
TooN webpage:
\code
	http://mi.eng.cam.ac.uk/~twd20/Numerics/index.html
\endcode
Download latest version from
\code
	http://mi.eng.cam.ac.uk/~er258/cvd/
\endcode
about the bottom of the page, and un-tgz in this directory. Documentation is at
\code
	http://mi.eng.cam.ac.uk/~er258/cvd/toon/html/
\endcode

@section Usage
Once compiled, the program should be used with this command line:
\code
	./toon
\endcode

@author PARP Research Group. University of Murcia, Spain.
*/

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <QDebug>

#include <qvgui/qvgui.h>
#include <qvcore/qvapplication.h>

#include <TooN/numerics.h>	// Basic TooN library
#include <TooN/numhelpers.h>	// If you want a few extras
#include <TooN/SVD.h>		// If you want singular value decompositions
#include <TooN/LU.h>		// If you want lu triangular decompositions
#include <TooN/SymEigen.h>	// If you want Eigen decomposition of symmetric matrices

class MyWorker: public QVWorker
	{
	public:
		MyWorker(QString name): QVWorker(name)	{}
		void iterate()
			{
			qDebug() << "MyWorker::worker()";
		
			Vector<3> v1((double[]){1,2,3});
			Matrix<3,4> M((double[]){1,2,3,4,5,6,7,8,9,10,11,12});
			Vector<4> v2((double[]){1,2,3,4});
		
			std::cout << "v1 = " << v1 << std::endl;
			std::cout << "M = " << std::endl << M ;
			std::cout << "v2 = " << v2 << std::endl;
		
			std::cout << "v1 * M * v2 = " << v1*M*v2 << std::endl;
		
			qDebug() << "MyWorker::worker(): <- return";
			}
	};

int main(int argc, char *argv[])
	{
	QVApplication app(argc, argv,
		"Example program for QVision library. Applies corner detection over an input video."
		);

	MyWorker worker("Corners Worker");

	QVGUI interface;

	return app.exec();
	}
