/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file cpustat.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <qstring.h>
#include <qdebug.h>
#include "cpustat.h"
#include <stdio.h>
#include <stdlib.h>
#include <iostream>
CpuStat::CpuStat(): doneFirstRound(false), showStats(false), flagNames(), lastTime(), rounds(0), totalRounds(0)
	{ lastTime.start(); };

void CpuStat::printStatsToConsole(bool enabled)		{ showStats = enabled; }
bool CpuStat::isActive()				{ return doneFirstRound; }
double *CpuStat::getLastMeans()				{ return mean; };
QStringList CpuStat::getFlagNames() const		{ return flagNames;	};

void CpuStat::step()
	{
	if (!flagNames.empty())
		{
		if(doneFirstRound)
			{
			for (int i = 0; i < flagNames.size(); ++i)
				{
				accumulated[i] += temporal[i];
				temporal[i] = 0;
				}
			rounds++;
			}
		else
			{
			doneFirstRound = true;		

			temporal = new double[flagNames.size()] ();
			accumulated = new double [flagNames.size()] ();
			totalAccumulated = new double [flagNames.size()] ();		
			mean = new double [flagNames.size()] ();

			for (int i=0; i< flagNames.size(); i++)
				accumulated[i] = mean[i] = temporal[i] = totalAccumulated[i] = 0;
			totalRounds = 0;
			}
		}

	lastTime.start();
	}

bool CpuStat::updateStats()
	{
	if (rounds == 0)
		return false;

	for (int i = 0; i < flagNames.size(); ++i)
		{
		mean[i] = accumulated[i] / rounds;
		totalAccumulated[i] += mean[i];
		}

	totalRounds++;

	if (showStats)
		{
		std::cout << "Cpu stats: Iteration " << totalRounds << " (steps elapsed " << rounds << ")." << std::endl;
		std::cout << "Cpu stats:\t\tAccum\tActual\tFlag name" << std::endl;
		for (int i = 0; i< flagNames.size(); ++i)
			std::cout << "CpuStat: stats:\t\t" << totalAccumulated[i]/(double)totalRounds << "\t" << mean[i] << "\t" << flagNames.at(i).toAscii().data() << std::endl;
		std::cout << "Cpu stats:" << std::endl;
		}

	for (int i = 0; i < flagNames.size(); ++i)
		accumulated[i] = 0;
	rounds = 0;
	return true;
	}

void CpuStat::setFlag(QString flagName)
	{
	int	elapsedTime = lastTime.elapsed();
	lastTime.start();

	if (doneFirstRound)		
		temporal[flagNames.lastIndexOf(flagName)] = elapsedTime;
	else	// we are in the first round still
		flagNames << flagName;
	}




